/***************************************************************************
                          bowl.h  -  description
                             -------------------
    begin                : Wed Mar 1 2000
    copyright            : (C) 2000 by Michael Speck
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOWL__H
#define BOWL__H

/**Becher fr die Tetris-Steine
  *@author Michael Speck
  */

class Wave;
struct Controls;

#include "hiscore.h"
#include "block.h"
#include "shrapnelllist.h"

#define LEVELNUMBER	21
struct LevelInit {
	float	per_msec; //speed
	int		lines; //how many lines do you need to proceed to this level
	int		score; //base score
};

//level action
#define LA_NONE		0
#define LA_ADDROW	1
#define LA_ADDBLOCK	2
//bowlcontents size
#define BC_W    10
#define BC_H    20
//offset in bowl in blocks
#define BC_X    1
#define BC_Y    4
struct AdvLevelInit {
	char	bowlcontents[BC_W][BC_H];
	int		line_score;
	int		delay; //action every delay ms
	int		action;
	char	dismantle_saves;
	int		strength; //used for add_row
};

//return values
#define BOWL_RET_NONE	    0
#define BOWL_RET_GAMEOVER	(1L<<0)
#define BOWL_RET_NEWLEVEL	(1L<<1) //not game over!!!
#define BOWL_RET_ENEMYHASLOST   (1L<<2)

class Bowl {
public: 
	Bowl(int bx, int by, SDL_Surface *scr, SDLSurface *bkgnd, SDLSurface * bl, HiScoreEntry *p, int ln, LevelInit *level_init, AdvLevelInit *adv_l_init, Controls *plyr, int anim_l, int trp, int hd, int w, ShrapnellList *s_list, SDL_Rect *rects, int *rect_num, int *tl, int s_all, int s_tet, int smth, int hlp, int bl_by_bl);
	~Bowl();
	void ClearBowl();
	int Update(int ms, char *keystate);
	void FirstBlock();
	void NextBlock();
	void HideBlock();
	void ShowBlock();
	void InsertBlock();
	void DrawBlocks();
	int  CheckLines();
	int  TooHigh();
	int  NewNextBlock();
	BlockInit* Next();
	void NextLevel();
	void AdvAction();
	int* RemLines();
	void AddShrapnells(int s_x, int *s_y = 0, int ln = -1);
	int X();
	void SetEnemy(Bowl *e);
	int  SendLines(int i[BOWL_W][BOWL_H], int *l_y, int num);
	int  ReceiveLines(int lines[4][BOWL_W], int num);
	void Dest_Hide();
	void Dest_Show();
	void Dest_Update();
private:
	SDL_Surface	*screen;
	SDLSurface	*backgnd;
	SDLSurface	*blocks;
	int 		bowl[BOWL_W+2][BOWL_PIX_H + BLOCKSIZE];
	int			bowl_x; //offset in surface
	int			bowl_y; // -"-
	int         bowl_vis_w; // in pixels
	int         bowl_vis_h; //
	BlockInit	*block_init;
	int			block_num;
	BlockInit	*next;
	Block		*block;
	HiScoreEntry	*player;
	LevelInit	*level_init;
	AdvLevelInit	*adv_l_init;
	int			line_offset;
	int			new_next;
	int			last_action; //time since last action in ms
	int			rem_lines;
	int			level_num;
	ShrapnellList	*shrapnells;
	SDL_Rect	*rects;
	int			*rect_num;
	int			anim_l;
	int			trp;
	int			hori_delay;
	Wave		*snd_explosion;
	Wave		*snd_rotate;
	Controls    *ctrls;
	Controls    *last_ctrls;
	int         *total_lines;
	Bowl        *enemy;
	int         wholes;
	int         send_all;
	int         send_tetris;
	int         smooth;
	// destination
	int         dest_x, dest_y; // not in pixels //
	int         dest_state; // rotation state //
	int         help;
};

#endif
