/***************************************************************************
                          game.cpp  -  description
                             -------------------
    begin                : Tue Feb 29 2000
    copyright            : (C) 2000 by Michael Speck
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <SDL/SDL.h>
#include <stdlib.h>
#include <string.h>
#include "game.h"
#include "defs.h"
#include "soundserver.h"

Game::Game()
{
	//video mode
	sdl = 0;
#ifdef SOUND	
	sdl = new SDL(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER);
#else
	sdl = new SDL(SDL_INIT_VIDEO | SDL_INIT_TIMER);
#endif	
	if (sdl->SetVideoMode(400, 325, 16, SDL_SWSURFACE) == SDL_ERROR)
		exit(EXIT_FAILURE);
	SDL_WM_SetCaption("LTris", 0);
		
	//load title
	title = new SDLSurface("title.bmp");
	if (title->Surface() == 0) {
		title = new SDLSurface(sdl->Width(), sdl->Height(), SDL_SWSURFACE);
		title->Fill(0, 0, title->Width(), title->Height(), 0x0);
	}
	f_copy = new SDLFont("font_s.sdlfnt");
	f_copy->SetAlignment(TA_X_RIGHT | TA_Y_BOTTOM);
	f_copy->Write(title->Surface(), title->Width()-3, title->Height()-1, "(C) 2000 Michael Speck");
    delete f_copy;
		
    //load logo
    logo = new SDLSurface("logo.bmp");
    if (logo->Surface() == 0)
    	exit(EXIT_FAILURE);
    logo_x = (sdl->Width() - logo->Width()) / 2;
    logo_y = 10;
    	
	//load fonts
	f_menu_w = new SDLFont("f_yellow.bmp", 32, 96, 10);
	f_menu_w->SetAlignment(TA_X_CENTER | TA_Y_CENTER);
	f_menu_b = new SDLFont("f_white.bmp", 32, 96, 10);
	f_menu_b->SetAlignment(TA_X_CENTER | TA_Y_CENTER);
	f_hiscore = new SDLFont("f_yellow.bmp", 32, 96, 10);
	f_hiscore->SetAlignment(TA_X_LEFT | TA_Y_TOP);
	f_enlgt_hiscore = new SDLFont("f_white.bmp", 32, 96, 10);
	f_enlgt_hiscore->SetAlignment(TA_X_LEFT | TA_Y_TOP);
	
	//restricted keys
	memset(val_ctrl_keys, 0, sizeof(val_ctrl_keys));
	for (int i = 32; i < 128; i++)
		val_ctrl_keys[i] = 1;
	val_ctrl_keys[SDLK_RETURN] = 0;
	val_ctrl_keys[SDLK_ESCAPE] = 0;
	val_ctrl_keys[SDLK_q] = 0;
	val_ctrl_keys[SDLK_p] = 0;
	val_ctrl_keys[SDLK_UP] = 1;
	val_ctrl_keys[SDLK_DOWN] = 1;
	val_ctrl_keys[SDLK_RIGHT] = 1;
	val_ctrl_keys[SDLK_LEFT] = 1;

	//tetris
	tetris = new Tetris(sdl);
		
	//create menumanager
	char    *str_gametype[] = {
	    "Normal",
	    "Advanced",
	    "Multiplayer"
	};
	char    *str_anim[] = {
	    "Off",
	    "Low",
	    "High"
	};
	char    *str_back[] = {
	    "Wood",
	    "Stone",
	    "Marble",
	    "Metal",
	    "Grid"
	};
	int cx = 200, cy = 240;
	mm = 0;
	mm = new MenuManager(sdl->Screen(), title, cx, cy, f_menu_w, f_menu_b, 10, val_ctrl_keys);
	//create menus
	Menu *menu[10];
	menu[0] = new Menu("main", 4);
	menu[1] = new Menu("options", 6);
	menu[2] = new Menu("controls", 6);
	menu[3] = new Menu("name", 5);
	menu[4] = new Menu("gamectrls", 8);
	menu[5] = new Menu("graphics", 8);
	menu[6] = new Menu("sound", 4);
	menu[7] = new Menu("player1", 7);
	menu[8] = new Menu("player2", 7);
	menu[9] = new Menu("multiplayer", 5);
    menu[0]->InsertItem(0, new MenuItem("New Game", menu[3]));
    menu[0]->InsertItem(1, new MenuItem("HiScore", AT_HISCORE));
    menu[0]->InsertItem(2, new MenuItem("Options", menu[1]));
    menu[0]->InsertItem(3, new MenuItem("Quit", AT_QUIT));
    menu[1]->InsertItem(0, new MenuItem("Game", menu[4]));
    menu[1]->InsertItem(1, new MenuItem("Controls", menu[2]));
    menu[1]->InsertItem(2, new MenuItem("Graphics", menu[5]));
#ifdef SOUND
    menu[1]->InsertItem(3, new MenuItem("Audio", menu[6]));
#else
    menu[1]->InsertItem(3, new MenuItem("Audio"));
#endif
    menu[1]->InsertItem(4, new MenuItem());
    menu[1]->InsertItem(5, new MenuItem("Back", menu[0]));
    menu[2]->InsertItem(0, new MenuItem("1st Player", menu[7]));
    menu[2]->InsertItem(1, new MenuItem("2nd Player", menu[8]));
	menu[2]->InsertItem(2, new MenuItem());
	menu[2]->InsertItem(3, new MenuItem("Delay(L&R):", &tetris->Setup()->hori_del, 0, 5));
	menu[2]->InsertItem(4, new MenuItem());
	menu[2]->InsertItem(5, new MenuItem("Back", menu[1]));
	menu[3]->InsertItem(0, new MenuItem("New Game", AT_NEWGAME));
	menu[3]->InsertItem(1, new MenuItem("1st Player", tetris->Setup()->name, 10));
	menu[3]->InsertItem(2, new MenuItem("2nd Player", tetris->Setup()->name2, 10));
	menu[3]->InsertItem(3, new MenuItem());
	menu[3]->InsertItem(4, new MenuItem("Back", menu[0]));
    menu[4]->InsertItem(0, new MenuItem("Game Mode:", &tetris->Setup()->gametype, str_gametype, 3));
    menu[4]->InsertItem(1, new MenuItem("Startlevel:", &tetris->Setup()->level, 0, 9));
    menu[4]->InsertItem(2, new MenuItem("Next Block:", &tetris->Setup()->next, "Off", "On"));
    menu[4]->InsertItem(3, new MenuItem("Help:", &tetris->Setup()->help, "Off", "On"));
	menu[4]->InsertItem(4, new MenuItem());
    menu[4]->InsertItem(5, new MenuItem("Multiplayer", menu[9]));
	menu[4]->InsertItem(6, new MenuItem());
	menu[4]->InsertItem(7, new MenuItem("Back", menu[1]));
    menu[5]->InsertItem(0, new MenuItem("Transparency:", &tetris->Setup()->transparency, "Off", "On"));
    menu[5]->InsertItem(1, new MenuItem("Animation (Game Over):", &tetris->Setup()->anim_go, "Off", "On"));
    menu[5]->InsertItem(2, new MenuItem("Animation (Line):", &tetris->Setup()->anim_l, str_anim, 3));
    menu[5]->InsertItem(3, new MenuItem("Smooth Horizontal Movement:", &tetris->Setup()->smooth_hori, "Off", "On"));
    menu[5]->InsertItem(4, new MenuItem("Block-By-Block Drop:", &tetris->Setup()->bl_by_bl, "Off", "On"));
    menu[5]->InsertItem(5, new MenuItem("Background:", &tetris->Setup()->bkgnd, str_back, 5));
	menu[5]->InsertItem(6, new MenuItem());
	menu[5]->InsertItem(7, new MenuItem("Back", menu[1]));
	menu[6]->InsertItem(0, new MenuItem("Sound:", &tetris->Setup()->snd_on, "Off", "On"));
	menu[6]->InsertItem(1, new MenuItem("Volume:", &tetris->Setup()->snd_vol, 1, 8));
	menu[6]->InsertItem(2, new MenuItem());
	menu[6]->InsertItem(3, new MenuItem("Back", menu[1]));
	menu[7]->InsertItem(0, new MenuItem("Left", &tetris->Setup()->plyr1.left, 0));
	menu[7]->InsertItem(1, new MenuItem("Right", &tetris->Setup()->plyr1.right, 0));
	menu[7]->InsertItem(2, new MenuItem("Rotate(L)", &tetris->Setup()->plyr1.rot_left, 0));
	menu[7]->InsertItem(3, new MenuItem("Rotate(R)", &tetris->Setup()->plyr1.rot_right, 0));
	menu[7]->InsertItem(4, new MenuItem("Down", &tetris->Setup()->plyr1.down, 0));
	menu[7]->InsertItem(5, new MenuItem());
	menu[7]->InsertItem(6, new MenuItem("Back", menu[2]));
	menu[8]->InsertItem(0, new MenuItem("Left", &tetris->Setup()->plyr2.left, 0));
	menu[8]->InsertItem(1, new MenuItem("Right", &tetris->Setup()->plyr2.right, 0));
	menu[8]->InsertItem(2, new MenuItem("Rotate(L)", &tetris->Setup()->plyr2.rot_left, 0));
	menu[8]->InsertItem(3, new MenuItem("Rotate(R)", &tetris->Setup()->plyr2.rot_right, 0));
	menu[8]->InsertItem(4, new MenuItem("Down", &tetris->Setup()->plyr2.down, 0));
	menu[8]->InsertItem(5, new MenuItem());
	menu[8]->InsertItem(6, new MenuItem("Back", menu[2]));
    menu[9]->InsertItem(0, new MenuItem("Wholes:", &tetris->Setup()->wholes, 1, 9));
    menu[9]->InsertItem(1, new MenuItem("Send All Lines:", &tetris->Setup()->send_all, "Off", "On"));
    menu[9]->InsertItem(2, new MenuItem("Always Send 'Tetris':", &tetris->Setup()->send_tetris, "Off", "On"));
	menu[9]->InsertItem(3, new MenuItem());
	menu[9]->InsertItem(4, new MenuItem("Back", menu[4]));
    //add to manager
    mm->InsertMenu(0, menu[0]);
    mm->InsertMenu(1, menu[1]);
    mm->InsertMenu(2, menu[2]);
    mm->InsertMenu(3, menu[3]);
    mm->InsertMenu(4, menu[4]);
    mm->InsertMenu(5, menu[5]);
    mm->InsertMenu(6, menu[6]);
    mm->InsertMenu(7, menu[7]);
    mm->InsertMenu(8, menu[8]);
    mm->InsertMenu(9, menu[9]);
    mm->Activate();

    //randomize
	timeb t;
	ftime(&t);
	srand((unsigned int)t.time);
	
#ifdef SOUND	
	//load waves
	snd_menu = new Wave("click.wav");
#endif	
}

Game::~Game()
{
	if (tetris) delete tetris;
	if (title) delete title;
	if (logo) delete logo;
	if (mm) delete mm;
	if (f_menu_w) delete f_menu_w;
	if (f_menu_b) delete f_menu_b;
	if (f_hiscore) delete f_hiscore;
	if (f_enlgt_hiscore) delete f_enlgt_hiscore;
#ifdef SOUND
	if (snd_menu) delete snd_menu;
#endif	
	if (sdl) delete sdl;
}

void Game::Run()
{
#ifdef SOUND
	sndsrv->Open(snd_menu->Spec());
#endif
	
	mm->Prepare();

	title->Blit(sdl->Screen(), 0, 0);
	SC_Trp_X(sdl->Screen()->pixels, sdl->Width(), logo_x, logo_y, logo->Surface()->pixels, logo->Width(), 0, 0, logo->Width(), logo->Height(), 60);
	
	sdl->UnDim();

	mm->CurMenu()->Show(1);
	//loop
	SDL_Event	event;
	int			go_on = 1;
	while (go_on && !fast_quit) {
		if (SDL_PollEvent(&event))
			if (event.type == SDL_KEYUP) {
#ifdef SOUND
				sndsrv->PlayWave(snd_menu);
#endif
				if (event.key.keysym.sym == SDLK_ESCAPE) break;
				switch (mm->KeyEvent(&event.key)) {
					case MR_CONTINUE:
						break;
					case AT_HISCORE:
						sdl->Dim();
						ShowHiScore(-1, -1);
						title->Blit(sdl->Screen(), 0, 0);
						SC_Trp_X(sdl->Screen()->pixels, sdl->Width(), logo_x, logo_y, logo->Surface()->pixels, logo->Width(), 0, 0, logo->Width(), logo->Height(), 60);
						sdl->UnDim();
						mm->CurMenu()->Show(1);
						break;
					case AT_QUIT:
						go_on = 0;
						break;
					case AT_NEWGAME:
						sdl->Dim();
						int rank = tetris->Run(); //game
						if (!fast_quit) {
						    if (rank != -1)
							    ShowHiScore(tetris->GameType(), rank);
						    title->Blit(sdl->Screen(), 0, 0);
						    SC_Trp_X(sdl->Screen()->pixels, sdl->Width(), logo_x, logo_y, logo->Surface()->pixels, logo->Width(), 0, 0, logo->Width(), logo->Height(), 60);
						    sdl->UnDim();
						    mm->CurMenu()->ClearState();
						    mm->SetCurMenu(0);
						    mm->CurMenu()->Prepare(0);
						    mm->CurMenu()->Show(1);
						}
						break;
				}
			}
			else
			    if (event.type == SDL_QUIT)
			        fast_quit = 1;
			
		if (fast_quit) break;
		
	 	//compute
 	    mm->CurMenu()->Compute();
 	    mm->CurMenu()->Update(1);
	 	
	 	SDL_Delay(8);
	
#ifdef SOUND
		sndsrv->Activate(tetris->Setup()->snd_on);
 		sndsrv->SetVolume(tetris->Setup()->snd_vol);
#endif
	}
	
    sdl->Dim();

#ifdef SOUND
	sndsrv->Close();
#endif
}

void Game::ShowHiScore(int h, int e)
{
	title->Blit(sdl->Screen(), 0, 0);
	sdl->UnDim();
	
	//write
	int i;
	int a_x = 50, a_y = 100;
	int level_off = 150, lines_off = 200, score_off = 250;
	int entry_h = f_hiscore->Height() + 2;
	char buffer[12];
	char headline[3][32] = {"HiScore 'Normal'", "HiScore 'Advanced'", "HiScore 'Multiplayer'"};
	HiScoreEntry	entry;
	SDLFont			*font;
	int a, b;
	if (h == -1) {
		a = 0; b = 2;
	}
	else
		a = b = h;
	for (int j = a; j <= b; j++) {
		f_hiscore->SetAlignment(TA_X_CENTER | TA_Y_CENTER);
		f_hiscore->Write(sdl->Screen(), sdl->Width() / 2, 50, headline[j]);
		f_hiscore->SetAlignment(0);
		for (i = 0; i < 10; i++) {
			if (j == h && i == e && h != -1)
				font = f_enlgt_hiscore;
			else
				font = f_hiscore;
			entry = tetris->GetHiScore(j)->Entry(i);
			font->Write(sdl->Screen(), a_x, a_y + i * entry_h, entry.name);
			sprintf(buffer, "%i", entry.level);
			font->Write(sdl->Screen(), a_x + level_off, a_y + i * entry_h, buffer);
			sprintf(buffer, "%i", entry.lines);
			font->Write(sdl->Screen(), a_x + lines_off, a_y + i * entry_h, buffer);
			sprintf(buffer, "%i", entry.score);
			font->Write(sdl->Screen(), a_x + score_off, a_y + i * entry_h, buffer);
		}
		sdl->Update();
		if (fast_quit || sdl->WaitForKey() == SDLK_ESCAPE || h != -1) break;
		if (j < 2) title->Blit(sdl->Screen(), 0, 0);
	}
		
	sdl->Dim();
}
