/***************************************************************************
                          tetris.cpp  -  description
                             -------------------
    begin                : Tue Feb 29 2000
    copyright            : (C) 2000 by Michael Speck
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tetris.h"
#include "preview.h"

Tetris::Tetris(SDL *sdl)
{
	//set sdl
	Tetris::sdl = sdl;
    memset(keystate, 0, sizeof(keystate));
    rect_num = 0;	
    new_level = 0;
    game_over = 0;
    always_full_refresh = 0;
    wait_for_key = 0;
    snapshot = 0;
    q_pct = 90;
    q_pms = -0.05;

	//setup init data
	/*config path*/
	cfg_path = new char[strlen(home_dir) + strlen(".ltris.cfg") + 2];
	sprintf(cfg_path, "%s/.ltris.cfg", home_dir);
	/*load init*/
	FILE	*f;
	if ((f = fopen(cfg_path, "r")) == 0) {
        init_data.level = 0;
		init_data.gametype = 0;
		init_data.next = 1;
		init_data.plyr1.left = SDLK_LEFT;
		init_data.plyr1.right = SDLK_RIGHT;
		init_data.plyr1.down = SDLK_DOWN;
		init_data.plyr1.rot_left = SDLK_UP;
		init_data.plyr1.rot_right = SDLK_SPACE;
		init_data.plyr2.left = 'a';
 		init_data.plyr2.right = 'd';
		init_data.plyr2.down = 's';
		init_data.plyr2.rot_left = 'w';
		init_data.plyr2.rot_right = 'e';
		strcpy(init_data.name, "Michael");
		strcpy(init_data.name2, "Sabine");
		init_data.anim_l = 2;
		init_data.anim_go = 1;
		init_data.transparency = 1;
		init_data.hori_del = 5;
		init_data.snd_on = 1;
		init_data.snd_vol = 6;
		init_data.wholes = 4;
		init_data.send_all = 0;
		init_data.send_tetris = 1;
		init_data.bkgnd = 0;
		init_data.smooth_hori = 0;
		init_data.help = 0;
		init_data.bl_by_bl = 0;
	}
	else {
		fread(&init_data, sizeof(InitData), 1, f);
		fclose(f);
	}
	
	//load hiscores
	hiscore[0] = new HiScore("highscore.0");
	hiscore[1] = new HiScore("highscore.1");
	hiscore[2] = new HiScore("highscore.2");
	
	//load font
	font = new SDLFont("f_white.bmp", 32, 96, 10);
		
	//load gfx
	blocks = new SDLSurface("blocks.bmp");
	quest_mark = new SDLSurface("quest.bmp");
	
	//create backgnd
	picture = new SDLSurface("bkgnd0.bmp");
	if (picture->Surface() == 0) {
		picture = new SDLSurface(sdl->Screen()->w, sdl->Screen()->h, SDL_SWSURFACE);
		picture->Fill(0, 0, picture->Width(), picture->Height(), 0x0);
	}
	backgnd = new SDLSurface(picture->Width(), picture->Height(), SDL_SWSURFACE);
	picture->Blit(backgnd, 0, 0);

	//create numbers
	int x_off = 220, y_off = 35;
	memset(&player, 0, sizeof(HiScoreEntry));
	highscore = new Number(sdl->Screen(), backgnd, font, x_off + 140, y_off, &val_hiscore, 1);
	score = new Number(sdl->Screen(), backgnd, font, x_off + 140, y_off + 25, &player.score, 1);
	score2 = new Number(sdl->Screen(), backgnd, font, x_off + 140, y_off + 25, &player2.score, 1);
	lines = new Number(sdl->Screen(), backgnd, font, x_off + 140, y_off + 50, &player.lines, 0.004);
    level = new Number(sdl->Screen(), backgnd, font, x_off + 140, y_off + 75, &player.level, 1);

	//bowl	
    bowl = bowl2 = 0;

	//preview
	preview = preview2 = 0;
	
	//level init
	int i;
	level_init[0][0].lines = 10;
	level_init[2][0].lines = 20;
	level_init[0][0].score = level_init[2][0].score = 100;
	level_init[0][0].per_msec = level_init[2][0].per_msec = 0.025;
	for (i = 1; i < LEVELNUMBER; i++) {
		level_init[0][i].lines = level_init[0][i-1].lines + 10;
		level_init[2][i].lines = level_init[2][i - 1].lines + 20;
		level_init[0][i].score = level_init[2][i].score = level_init[0][i-1].score + 100;
	}
	for (i = 1; i < 3; i++)
		level_init[0][i].per_msec = level_init[2][i].per_msec = level_init[0][i-1].per_msec + level_init[0][0].per_msec;
	for (i = 3; i < LEVELNUMBER; i++)
		level_init[0][i].per_msec = level_init[2][i].per_msec = level_init[0][i-1].per_msec + level_init[0][0].per_msec/3;

	//set advanced init
	char interior[] = { //abcdefg = block id
		/* 0 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"      aa  "
		" ffffffff "
		" cccccccc "
		" hhhhhhhh "
		" hhhhhhhh "
		
		/* 1 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		" aaa  h   "
		" a a hhh  "
		" aaaaaaaa "
		" aiiaaiia "
		"  ii  ii  "
		
		/* 2 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"i i    i i"
		"iii ii iii"
		" iiiiiiii "
		" iiiiiiii "
		" iii  iii "
		" iii  iii "
		" iii  iii "
		
		/* 3 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		" hhhh     "
		" dddd     "
		" hhhh     "
		" dddd hhhh"
		" hhhh dddd"
		" dddd hhhh"
	
		/* 4 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"     ccc  "
		"     ccccc"
		"     ccccc"
		"     ccccc"
		"      ccc "
		"  cc ccccc"
		"cccc ccccc"
	
		/* 5 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"  fffffff "
		" fcccccc  "
		" fiiiiii  "
		" fcccccc  "
		" fiiiiii  "
		"  fffffff "
	
		/* 6 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"  c    c  "
		"  c    c  "
		"  c    c  "
		"  c    c  "
		"  ceeeec  "
		"  ceeeec  "
		"  ceeeec  "
		"  cccccc  "
	
		/* 7 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"   dddd   "
		"  dddddd  "
		" dddddddd "
		"dd  dd  dd"
		"dd  dd  dd"
		"d dddddd d"
		"dd      dd"
		" dd    dd "
		"  dddddd  "
		"   dddd   "
	
		/* 8 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"c  c c  cc"
		"cccc c c  "
		"c  c c c  "
		"c  c c c  "
		"c  c c  cc"
	
		/* 9 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		" aaa  aaa "
		"aa aaaa aa"
		"aa      aa"
		"aa      aa"
		" aa    aa "
		"  aa  aa  "
		"   aaaa   "
		"    aa    "
	
		/* 10 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		" a a  a a "
		" a a  a a "
		" a a  a a "
		" hhh  hhh "
		"  hhhhhh  "
		"    hh    "
		"   hhhh   "
		"  hhhhhh  "

		/* 11*/
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"   bbbb   "
		"  b bb b  "
		" b  bb  b "
		"b   bb   b"
		"b   bb   b"
		"b  bbbb  b"
		"b bb  bb b"
		" bb    bb "
		"  b    b  "
		"   bbbb   "
	
		/* 12 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"   hhhh   "
		"    hhh   "
		" hh hh  h "
		"hhh h  hhh"
		"      hhhh"
		"fff  ff   "
		"ffff ff ff"
		"ffff    ff"
		"fffff  fff"
	
		/* 13 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		" faaa     "
		" faaaaaa  "
		" faaaaaa  "
		" faaaaaa  "
		" f   aaa  "
		" f        "
		" f        "
		" f        "
		" f        "
	
		/* 14 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"    cc    "
		"   cccc   "
		"   cccc   "
		"  cc  cc  "
		"  cc  cc  "
		"  cccccc  "
		"  cccccc  "
		"  cccccc  "
		"   cccc   "
		"  acccca  "
		" aaccccaa "
		" aa cc aa "
	
		/* 15 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"   aca    "
		"  acaca   "
		" aca aca  "
		" ca   ac  "
		" ac   ca  "
		" ca       "
		" ac       "
		" ca       "
		" ac       "
		" ca       "
	
		/* 16 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		" e e e e e"
		"eeeee eeee"
		"e  eeee e "
		"e  e eeeee"
		"eeeeee  e "
		"ee eee  ee"
		" eeeeeeeee"
	
		/* 17 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"      d   "
		"    caad  "
		"   cc d   "
		"  iiii    "
		" iiiiii   "
		" iiiiii   "
		" iiiiii   "
		" iiiiii   "
		"  iiii    "
	
		/* 18 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"    aa    "
		"   aaaa   "
		"  aaaaaa  "
		"  aaaaaa  "
		"   aaaa   "
		"    aa    "
		"   i  i   "
		"   i  i   "
		"    ff    "
		"   cffc   "
	
		/* 19 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
	
		/* 20 */
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
		"          "
	};
	//puh!!!
	
	memcpy(level_init[1], level_init[0], sizeof(LevelInit)*LEVELNUMBER);
	float base = 0.025;
	level_init[1][7].per_msec = level_init[0][5].per_msec;
    for (i = 8; i < LEVELNUMBER; i++)
    	level_init[1][i].per_msec = level_init[1][i-1].per_msec + base/3;
	for (i = 0; i < LEVELNUMBER; i++) {
		adv_l_init[i].line_score = i*10 + 10;
		for (int a = 0; a < BC_W; a++)
			for (int b = 0; b < BC_H; b++)
				adv_l_init[i].bowlcontents[a][b] = interior[i*BC_W*BC_H + b*BC_W + a];
	}
	for (i = 0; i < 7; i++)
		adv_l_init[i].action = LA_NONE;
	for (i = 7; i <= 12; i++)
		adv_l_init[i].action = LA_ADDBLOCK;
	for (i = 13; i <= 20; i++)
		adv_l_init[i].action = LA_ADDROW;
	for (i = 7; i <= 20; i++)
		adv_l_init[i].dismantle_saves = 1;
		
	for (i = 7; i <= 12; i++)
		adv_l_init[i].delay = 2000 + (12-i)*500;
	adv_l_init[12].dismantle_saves = 0;
		
	for (i = 13; i <= 19; i++) {
		adv_l_init[i].delay = 1500 + (20-i)*500;
		adv_l_init[i].strength = i-12; //wholes
	}
	adv_l_init[19].delay = 3000;
	adv_l_init[19].dismantle_saves = 0;
	adv_l_init[20].dismantle_saves = 0;
	
	//shrapnell list
	shrapnells = shrapnells2 = 0;
}

Tetris::~Tetris()
{
	hiscore[0]->Save(); delete hiscore[0];
	hiscore[1]->Save(); delete hiscore[1];
	hiscore[2]->Save(); delete hiscore[2];
	if (backgnd) delete backgnd;
	if (score) delete score;
	if (score2) delete score2;
	if (highscore) delete highscore;
	if (level) delete level;
	if (lines) delete lines;
	if (font) delete font;
	if (bowl) delete bowl;
	if (bowl2) delete bowl2;
	if (blocks) delete blocks;
	if (shrapnells) delete shrapnells;
	if (shrapnells2) delete shrapnells2;
	if (quest_mark) delete quest_mark;
	if (preview) delete preview;
	if (preview2) delete preview2;
	
	/*save init*/
	FILE	*f = fopen(cfg_path, "w");
    fwrite(&init_data, sizeof(InitData), 1, f);
	fclose(f);
	if (cfg_path) delete cfg_path;
}

InitData* Tetris::Setup()
{
	return &init_data;
}

HiScore* Tetris::GetHiScore(int i)
{
	i = (i < 0) ? 0 : (i > 2) ? 2 : i;
	return hiscore[i];
}

int Tetris::Run()
{
    if (init_data.gametype == 2)
        sdl->SetVideoMode(540, 355, 16, SDL_SWSURFACE);
    ReloadBkgnd();
    backgnd->Blit(sdl->Screen(), 0, 0);
    sdl->UnDim();
	
	//init game
	winner = 0;
    InitGame();
    score->Show(rects, &rect_num);
    lines->Show(rects, &rect_num);
    level->Show(rects, &rect_num);
	
	//main loop
	SDL_Event	event;
	int			go_on = 1;
	int         last_call, call;
	int 		ms = 0;
	
	last_call = SDL_GetTicks();
	while (go_on && !fast_quit) {
		if (SDL_PollEvent(&event))
			//check keystates
			switch (event.type) {
			    case SDL_QUIT:
			        fast_quit = 1;
			        break;
				case SDL_KEYUP:
					if (event.key.keysym.sym == SDLK_TAB)
						SnapShot();
					if (wait_for_key && ((bowl2 == 0 && shrapnells->Number() == 0) || (shrapnells->Number() == 0 && shrapnells2->Number() == 0)))
						go_on = 0;
					if (event.key.keysym.sym == SDLK_p && !game_over) {
						PauseGame();
						last_call = SDL_GetTicks();
					}
					if (event.key.keysym.sym == SDLK_ESCAPE) {
					    go_on = 0;
					    if (!bowl2)
					        winner = &player;
					}
					keystate[event.key.keysym.sym] = 0;
					break;
				case SDL_KEYDOWN:
					keystate[event.key.keysym.sym] = 1;
					break;
				default:
					break;
			}
			
		//time since last loop
		call = SDL_GetTicks();
		ms = call - last_call;
		last_call = call;
		if (ms == 0) {
		    ms = 1;
		    SDL_Delay(1);
		}
		
		//update
		score->Hide(rects, &rect_num);
		score2->Hide(rects, &rect_num);
		highscore->Hide(rects, &rect_num);
		lines->Hide(rects, &rect_num);
		level->Hide(rects, &rect_num);
		preview->Hide();
		if (bowl2)
		    preview2->Hide();
		if (!new_level && !game_over && !wait_for_key) {
			int bowl_ret = bowl->Update(ms, keystate);
			int bowl2_ret = BOWL_RET_NONE;
			if (bowl2)
			    bowl2_ret = bowl2->Update(ms, keystate);

			//check game over
			if (bowl2 != 0) {
			    if (bowl_ret & BOWL_RET_ENEMYHASLOST)
			        bowl2_ret = bowl2_ret | BOWL_RET_GAMEOVER;
			    if (bowl2_ret & BOWL_RET_ENEMYHASLOST)
			        bowl_ret = bowl_ret | BOWL_RET_GAMEOVER;
			}

			if (bowl_ret & BOWL_RET_NEWLEVEL)
			    new_level = 1;
			else
			    if (bowl_ret & BOWL_RET_GAMEOVER || bowl2_ret & BOWL_RET_GAMEOVER) {
			        //game is over
			        game_over = 1;
			
			        //get winner
			        if (bowl2 == 0)
			            winner = &player;
			        else
			            if (bowl_ret & BOWL_RET_GAMEOVER && !bowl2_ret & BOWL_RET_GAMEOVER)
			                winner = &player2;
			            else
			                if (bowl2_ret & BOWL_RET_GAMEOVER && !bowl_ret & BOWL_RET_GAMEOVER)
			                    winner = &player;

			        //effects
					font->SetAlignment(TA_X_CENTER | TA_Y_CENTER);
			        if (init_data.anim_go) {
					    always_full_refresh = 1;
			            if (bowl_ret & BOWL_RET_GAMEOVER) {
						    bowl->AddShrapnells(bowl->X());
						    bowl->ClearBowl();
						    bowl->DrawBlocks();
						    font->Write(sdl->Screen(), bowl->X() + BOWL_PIX_W / 2, 200, "Game Over", TEXT_TRP);
			    			shrapnells->GetBackgnd();
        					player.score += (int)(player.score * (bonus_nonext+bonus_startlevel));
			            }
			            if (bowl2 && bowl2_ret & BOWL_RET_GAMEOVER) {
						    bowl2->AddShrapnells(bowl2->X());
						    bowl2->ClearBowl();
						    bowl2->DrawBlocks();
						    font->Write(sdl->Screen(), bowl2->X() + BOWL_PIX_W / 2, 200, "Game Over", TEXT_TRP);
			    			shrapnells2->GetBackgnd();
        					player2.score += (int)(player2.score * (bonus_nonext+bonus_startlevel));
			            }
			        }
			        else {
			            if (bowl_ret & BOWL_RET_GAMEOVER)
					        font->Write(sdl->Screen(), bowl->X() + BOWL_PIX_W / 2, 200, "Game Over", TEXT_TRP);
			            if (bowl2_ret & BOWL_RET_GAMEOVER)
					        font->Write(sdl->Screen(), bowl2->X() + BOWL_PIX_W / 2, 200, "Game Over", TEXT_TRP);
                    }					
					font->SetAlignment(TA_X_RIGHT | TA_Y_TOP);
			    }
		}
		else {
			shrapnells->Hide();
			shrapnells->Update(ms);
			shrapnells->GetBackgnd();
			if (shrapnells->Number() == 0) {
				if (new_level) {
					new_level = 0;
					bowl->NextBlock();
				}
				else
					wait_for_key = 1;
			}
			if (bowl2) {
			    shrapnells2->Hide();
			    shrapnells2->Update(ms);
			    shrapnells2->GetBackgnd();
			    if (shrapnells2->Number() == 0)
			        wait_for_key = 1;
			}
		}
		if (bowl2)
		    highest_level = player.level > player2.level ? player.level : player2.level;
		score->Update(ms);
		score2->Update(ms);
		highscore->Update(ms);
		lines->Update(ms);
		level->Update(ms);
		score->Show(rects, &rect_num);
		score2->Show(rects, &rect_num);
		highscore->Show(rects, &rect_num);
		lines->Show(rects, &rect_num);
		level->Show(rects, &rect_num);
		//compute trancparency
		q_pct += (float)ms * q_pms;
		if ((int)q_pct > 90) {
			q_pms *= -1;
			q_pct = 90;
		}
		if ((int)q_pct < 20) {
			q_pms *= -1;
			q_pct = 20;
		}
		preview->Update(rects, &rect_num);
		if (bowl2)
		    preview2->Update(rects, &rect_num);
		shrapnells->Show(rects, &rect_num);
		if (bowl2)
		    shrapnells2->Show(rects, &rect_num);
		
		Refresh();
	}
	
	sdl->Dim();
    if (init_data.gametype == 2 && !fast_quit)
        sdl->SetVideoMode(400, 325, 16, SDL_SWSURFACE);

#define DEL(ptr) {delete ptr; ptr = 0;}
    //delete stuff
	if (shrapnells) DEL(shrapnells);
	if (shrapnells2) DEL(shrapnells2);
	if (bowl) DEL(bowl);
	if (preview) DEL(preview);
	if (bowl2) DEL(bowl2)
	if (preview2) DEL(preview2);
#undef DEL(ptr)
		
	//game over, check hiscore entry
	if (winner)
	    return hiscore[init_data.gametype]->CheckEntry(winner);
    return -1;	
}

void Tetris::InitGame()
{
	//reset
	total_lines = 0;
	player.lines = player.score = 0;
	player2.lines = player2.score = 0;
	if (init_data.gametype == 1) {
		player.level = 0;
		player2.level = 0;
    }
	else {
		player.level = init_data.level;
		player2.level = init_data.level;
	}
	highest_level = player.level;
	strcpy(player.name, init_data.name);
	strcpy(player2.name, init_data.name2);
	val_hiscore = hiscore[init_data.gametype]->Entry(0).score;
	
	//flags
	new_level = game_over = always_full_refresh = wait_for_key = 0;
	
	//bonus
	if (!init_data.next)
		bonus_nonext = 0.15;
	else
		bonus_nonext = 0;
	bonus_startlevel = 0.015 * init_data.level;
	if (init_data.gametype == 2)
	    bonus_startlevel = 0;

	//special inits
	shrapnells = new ShrapnellList();
	shrapnells2 = new ShrapnellList();
	
	if (init_data.gametype != 2) {
	    //normal || advanced
	    AdvLevelInit *adv = 0;
	    if (init_data.gametype == 1) adv = adv_l_init;
   		bowl = new Bowl(20, -BLOCKSIZE*4, sdl->Screen(), backgnd, blocks, &player, LEVELNUMBER, level_init[init_data.gametype], adv, &init_data.plyr1, init_data.anim_l, init_data.transparency, init_data.hori_del, init_data.wholes, shrapnells, rects, &rect_num, 0, init_data.send_all, init_data.send_tetris, init_data.smooth_hori, init_data.help, init_data.bl_by_bl);
    	preview = new Preview(250, 200, 80, 80, sdl, backgnd, blocks, bowl, init_data.transparency);
    	if (!init_data.next)
    	    preview->IsUnknown(quest_mark, &q_pct);
    	InitInfoDisplay();
	}
	else {
	    //multiplayer
    	bowl = new Bowl(20, -BLOCKSIZE*4, sdl->Screen(), backgnd, blocks, &player, LEVELNUMBER, level_init[init_data.gametype], 0, &init_data.plyr1, init_data.anim_l, init_data.transparency, init_data.hori_del, init_data.wholes, shrapnells, rects, &rect_num, &total_lines, init_data.send_all, init_data.send_tetris, init_data.smooth_hori, init_data.help, init_data.bl_by_bl);
    	bowl2 = new Bowl(sdl->Width() - BOWL_PIX_W - 20, -BLOCKSIZE*4, sdl->Screen(), backgnd, blocks, &player2, LEVELNUMBER, level_init[init_data.gametype], 0, &init_data.plyr2, init_data.anim_l, init_data.transparency, init_data.hori_del, init_data.wholes, shrapnells2, rects, &rect_num, &total_lines, init_data.send_all, init_data.send_tetris, init_data.smooth_hori, init_data.help, init_data.bl_by_bl);
    	preview = new Preview(190, 20, 80, 80, sdl, backgnd, blocks, bowl, init_data.transparency);
    	preview2 = new Preview(sdl->Width() - BOWL_PIX_W - 20 - 90, 225, 80, 80, sdl, backgnd, blocks, bowl2, init_data.transparency);
    	if (!init_data.next) {
    	    preview->IsUnknown(quest_mark, &q_pct);
    	    preview2->IsUnknown(quest_mark, &q_pct);
    	}
    	//info
    	InitMultiInfo();
    	//connect
    	bowl->SetEnemy(bowl2);
    	bowl2->SetEnemy(bowl);
	}
	backgnd->Blit(sdl->Screen(), 0, 0);
	bowl->ClearBowl();
	bowl->FirstBlock();
	if (bowl2) {
	    bowl2->ClearBowl();
	    bowl2->FirstBlock();
	}
	FullRefresh();
}

void Tetris::Refresh()
{
	if (rect_num == 0) return;
	if (rect_num == DRAWRECTS || always_full_refresh) {
		sdl->Update();
		rect_num = 0;
		return;
	}
	SDL_UpdateRects(sdl->Screen(), rect_num, rects);
	rect_num = 0;
}

void Tetris::FullRefresh()
{
	SDL_UpdateRect(sdl->Screen(), 0, 0, sdl->Width(), sdl->Height());
}

int Tetris::GameType()
{
	return init_data.gametype;
}

void Tetris::PauseGame()
{
	SDLSurface	*buffer = new SDLSurface(sdl->Width(), sdl->Height(), SDL_SWSURFACE);
	sdl->Blit(buffer, 0, 0);
	sdl->Dim();
	
	//pause gimmicks
	SDL_Event	event;
	int			x, y, w, h, pct = 0, p_dir = 1;
	font->SetAlignment(TA_X_CENTER | TA_Y_CENTER);
	font->Write(sdl->Screen(), sdl->Width()/ 2, sdl->Height() / 2, "Pause", pct);
	x = font->LastX();
	y = font->LastY();
	w = font->LastW();
	h = font->LastH();
	font->SetAlignment(TA_X_LEFT | TA_Y_TOP);
	while (1) {
		SD_Box(sdl->Screen()->pixels, sdl->Width(), x, y, w, h, 0);
		if (pct == 100)	p_dir = -1;
		if (pct == 0) p_dir = 1;
		pct += p_dir * 2;
		font->Write(sdl->Screen(), x, y, "Pause", pct);
		sdl->Update(x, y, w, h);
		SDL_Delay(50);
		SDL_PollEvent(&event);
		if (event.type == SDL_KEYUP) break;
	}
    font->SetAlignment(TA_X_RIGHT | TA_Y_TOP);
	
	buffer->Blit(sdl->Screen(), 0, 0);
	sdl->UnDim();
}

void Tetris::SnapShot()
{
	char filename[32];
	sprintf(filename, "snapshot_%i.bmp", snapshot++);
	SDL_SaveBMP(sdl->Screen(), filename);
}

void Tetris::InitInfoDisplay()
{
    //normal or advanced
    //frame
	int x_off = 210, y_off = 20, w = 160, h = 20 + 4*25;
    font->SetAlignment(TA_X_LEFT | TA_Y_TOP);
    DrawFrame(x_off, y_off, w, h);
	//text
	font->Write(backgnd->Surface(), x_off + 5, y_off + 15, "HiScore:", TEXT_TRP);
	font->Write(backgnd->Surface(), x_off + 5, y_off + 25 + 15, "Score:", TEXT_TRP);
	font->Write(backgnd->Surface(), x_off + 5, y_off + 50 + 15, "Lines:", TEXT_TRP);
	font->Write(backgnd->Surface(), x_off + 5, y_off + 75 + 15, "Level:", TEXT_TRP);
    font->SetAlignment(TA_X_RIGHT | TA_Y_TOP);
    //reset
    highscore->Move(x_off + 155, y_off + 15);
	highscore->Reset();
    score->Move(x_off + 155, y_off + 25 + 15);
	score->Reset();
    level->Move(x_off + 155, y_off + 75 + 15);
    level->SetPtr(&player.level);
	level->Reset();
	if (init_data.gametype == 0)
	    lines->SetPtr(&player.lines);
	else
	    lines->SetPtr(bowl->RemLines());
    lines->Move(x_off + 155, y_off + 50 + 15);
    lines->Reset();
    score2->SetActive(0);
}

void Tetris::InitMultiInfo()
{
    //multiplayer
    //frame
	int x_off = 190, y_off = 115, w = 160, h = 20 + 3*25;
    font->SetAlignment(TA_X_LEFT | TA_Y_TOP);
    DrawFrame(x_off, y_off, w, h);
	//text
	font->Write(backgnd->Surface(), x_off + 5, y_off + 15, "HiScore:", TEXT_TRP);
	font->Write(backgnd->Surface(), x_off + 5, y_off + 25 + 15, "Lines:", TEXT_TRP);
	font->Write(backgnd->Surface(), x_off + 5, y_off + 50 + 15, "Level:", TEXT_TRP);
    //reset
    highscore->Move(x_off + 155, y_off + 15);
    highscore->Reset();
    lines->Move(x_off + 155, y_off + 25 + 15);
    lines->SetPtr(&total_lines);
    lines->Reset();
    level->Move(x_off + 155, y_off + 50 + 15);
    level->SetPtr(&highest_level);
    level->Reset();
    score->Move(bowl->X() + BOWL_PIX_W - 5, sdl->Height() - 28);
    score->Reset();
    score2->SetActive(1);
    score2->Move(bowl2->X() + BOWL_PIX_W - 5, sdl->Height() - 28);
    score2->Reset();

    //name & score frames
    DrawFrame(bowl->X(), 320, 160, 30);
    DrawFrame(bowl2->X(), 320, 160, 30);

	//add names
	// 7 chars displayed at maximum //
	char n[8];
	font->SetAlignment(TA_X_LEFT | TA_Y_TOP);
	if (strlen(init_data.name) > 7) {
	    strncpy(n, init_data.name, 6);
	    n[6] = '.';
	    n[7] = 0;
	}
	else
	    strcpy(n, init_data.name);
	font->Write(backgnd->Surface(), bowl->X() + 5, sdl->Height() - 28, n, TEXT_TRP);
	if (strlen(init_data.name2) > 7) {
	    strncpy(n, init_data.name2, 6);
	    n[6] = '.';
	    n[7] = 0;
	}
	else
	    strcpy(n, init_data.name2);
	font->Write(backgnd->Surface(), bowl2->X() + 5, sdl->Height() - 28, n, TEXT_TRP);
	
	font->SetAlignment(TA_X_RIGHT | TA_Y_TOP);
}

void Tetris::DrawFrame(int px, int py, int pw, int ph)
{
    SD_CBx(backgnd->Surface()->pixels, backgnd->Width(), px, py, pw, ph, 0x0, BOWL_DARK);
	SD_Rct(backgnd->Surface()->pixels, backgnd->Width(), px, py, pw - 1, ph - 1, BC_NRML);
	SD_Rct(backgnd->Surface()->pixels, backgnd->Width(), px + 1, py + 1, pw - 1, ph - 1, BC_DARK);
}

void Tetris::ReloadBkgnd()
{
    if (picture) delete picture;
    char name[32];
    sprintf(name, "bkgnd%i.bmp", init_data.bkgnd);
	picture = new SDLSurface(name);
	if (picture->Surface() == 0) {
		picture = new SDLSurface(sdl->Screen()->w, sdl->Screen()->h, SDL_SWSURFACE);
		picture->Fill(0, 0, picture->Width(), picture->Height(), 0x0);
	}
	picture->Blit(backgnd, 0, 0);
}
